@@ -1,6 +1,7 @@
-/* Copyright (c) 1997-2018
-   Ewgenij Gawrilow, Michael Joswig (Technische Universitaet Berlin, Germany)
-   http://www.polymake.org
+/* Copyright (c) 1997-2022
+   Ewgenij Gawrilow, Michael Joswig, and the polymake team
+   Technische Universität Berlin, Germany
+   https://polymake.org
 
    This program is free software; you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the
@@ -19,6 +20,28 @@
 #include <cstdlib>
 
 namespace pm { namespace perl {
+namespace {
+
+template <typename PerlIV>
+constexpr
+std::enable_if_t<(sizeof(PerlIV) <= sizeof(Int)), Int>
+Int_from_perlIV(PerlIV iv, const char*) noexcept
+{
+   return iv;
+}
+
+// there are perls out in the wild built with IV=long long in 32bit mode
+template <typename PerlIV>
+std::enable_if_t<(sizeof(PerlIV) > sizeof(Int)), Int>
+Int_from_perlIV(PerlIV iv, const char* msg)
+{
+   const Int result = static_cast<Int>(iv);
+   if (static_cast<PerlIV>(result) != iv)
+      throw std::runtime_error(std::string(msg) + " exceeds 32-bit integral limit; consider using polymake built for 64-bit architecture");
+   return result;
+}
+
+}
 
 SVHolder::SVHolder()
 {
@@ -85,34 +108,34 @@ SV* Scalar::const_int(int i)
    return sv;
 }
 
-int Scalar::convert_to_int(SV* sv)
+Int Scalar::convert_to_Int(SV* sv)
 {
-   MAGIC* mg=SvMAGIC(SvRV(sv));
+   MAGIC* mg = SvMAGIC(SvRV(sv));
    const glue::scalar_vtbl* t = reinterpret_cast<const glue::scalar_vtbl*>(mg->mg_virtual);
-   return (t->to_int)(mg->mg_ptr);
+   return (t->to_Int)(mg->mg_ptr);
 }
 
-double Scalar::convert_to_float(SV* sv)
+double Scalar::convert_to_Float(SV* sv)
 {
-   MAGIC* mg=SvMAGIC(SvRV(sv));
+   MAGIC* mg = SvMAGIC(SvRV(sv));
    const glue::scalar_vtbl* t = reinterpret_cast<const glue::scalar_vtbl*>(mg->mg_virtual);
-   return (t->to_float)(mg->mg_ptr);
+   return (t->to_Float)(mg->mg_ptr);
 }
 
-SV* ArrayHolder::init_me(int size)
+SV* ArrayHolder::init_me(Int size)
 {
    dTHX;
    AV* av = newAV();
-   if (size > 0) av_extend(av, size - 1);
+   if (size > 0) av_extend(av, size-1);
    return newRV_noinc((SV*)av);
 }
 
-void ArrayHolder::upgrade(int size)
+void ArrayHolder::upgrade(Int size)
 {
    dTHX;
    if (!SvROK(sv)) {
       AV* av = newAV();
-      if (size > 0) av_extend(av, size - 1);
+      if (size > 0) av_extend(av, size-1);
       (void)SvUPGRADE(sv, SVt_RV);
       SvRV(sv) = (SV*)av;
       SvROK_on(sv);
@@ -130,13 +153,13 @@ void ArrayHolder::set_contains_aliases()
    AvREAL_off((AV*)SvRV(sv));
 }
 
-int ArrayHolder::size() const
+Int ArrayHolder::size() const
 {
    dTHX;
-   return AvFILL((AV*)SvRV(sv)) + 1;
+   return AvFILL((AV*)SvRV(sv))+1;
 }
 
-SV* ArrayHolder::operator[] (int i) const
+SV* ArrayHolder::operator[] (Int i) const
 {
    dTHX;
    return *av_fetch((AV*)SvRV(sv), i, TRUE);
@@ -162,7 +185,7 @@ SV* ArrayHolder::pop()
 void ArrayHolder::unshift(SV* x)
 {
    dTHX;
-   AV* const ary=(AV*)SvRV(sv);
+   AV* const ary = (AV*)SvRV(sv);
    av_unshift(ary, 1);
    av_store(ary, 0, x);
 }
@@ -173,7 +196,7 @@ void ArrayHolder::push(SV* x)
    av_push((AV*)SvRV(sv), x);
 }
 
-void ArrayHolder::resize(int size)
+void ArrayHolder::resize(Int size)
 {
    dTHX;
    av_fill((AV*)SvRV(sv), size-1);
@@ -194,13 +217,13 @@ void HashHolder::verify()
 bool HashHolder::exists(const AnyString& key) const
 {
    dTHX;
-   return hv_exists((HV*)SvRV(sv), key.ptr, key.len);
+   return hv_exists((HV*)SvRV(sv), key.ptr, I32(key.len));
 }
 
 SV* HashHolder::fetch(const AnyString& key, bool create) const
 {
    dTHX;
-   SV** valp = hv_fetch((HV*)SvRV(sv), key.ptr, key.len, create);
+   SV** valp = hv_fetch((HV*)SvRV(sv), key.ptr, I32(key.len), create);
    return valp ? *valp : &PL_sv_undef;
 }
 
@@ -217,35 +240,17 @@ ListValueInputBase::ListValueInputBase(SV* sv)
       const bool is_magic = SvMAGICAL(arr_or_hash) != 0;
       if (SvTYPE(arr_or_hash) == SVt_PVAV) {
          AV* av = (AV*)arr_or_hash;
-         size_ = (is_magic ? av_len(av) : AvFILLp(av)) + 1;
-         // TODO: remove this when XML parser vanishes
-         if (MAGIC* mg = glue::array_flags_magic(aTHX_ arr_or_hash)) {
-            if (mg->mg_len >= 0 && mg->mg_obj && SvPOKp(mg->mg_obj)) {
-               if (SvCUR(mg->mg_obj) == 3 && !strncmp(SvPVX(mg->mg_obj), "dim", 3)) {
-                  dim_ = mg->mg_len;
-                  sparse_ = true;
-               } else if (SvCUR(mg->mg_obj) == 4 && !strncmp(SvPVX(mg->mg_obj), "cols", 4)) {
-                  cols_ = mg->mg_len;
-               }
-            }
-         } else if (!is_magic) {
+         size_ = (is_magic ? av_len(av) : AvFILLp(av))+1;
+         if (!is_magic) {
             if (size_ > 0) {
-               SV* last = AvARRAY(av)[size_ - 1];
-               if (SvOBJECT(av)) {
-                  if (SvSTASH(av) == glue::Serializer_Sparse_stash) {
-                     sparse_ = true;
-                     if (size_ % 2 == 1 && SvIOK(last)) {
-                        dim_ = SvUVX(last);
-                        --size_;
-                     }
-                  }
-               } else if (SvROK(last)) {
+               SV* last = AvARRAY(av)[size_-1];
+               if (!SvOBJECT(av) && SvROK(last)) {
                   last = SvRV(last);
                   if (SvTYPE(last) == SVt_PVHV && !SvOBJECT(last) && !SvMAGICAL(last)) {
                      HV* attrs = (HV*)last;
                      SV** colsp;
                      if (HvUSEDKEYS(attrs) == 1 && (colsp = hv_fetch(attrs, "cols", 4, FALSE))) {
-                        cols_ = SvIV(*colsp);
+                        cols_ = Int_from_perlIV(SvIV(*colsp), "number of columns");
                         --size_;
                      }
                   }
@@ -256,24 +261,37 @@ ListValueInputBase::ListValueInputBase(SV* sv)
          }
          return;
       }
-      if (!is_magic && SvTYPE(arr_or_hash) == SVt_PVHV) {
+      if (SvTYPE(arr_or_hash) == SVt_PVHV) {
          HV* hv = (HV*)arr_or_hash;
-         sparse_ = true;
-         UV dim_val;
-         if ((dim_sv = hv_delete_ent(hv, glue::Serializer_Sparse_dim_key, 0, SvSHARED_HASH(glue::Serializer_Sparse_dim_key)))) {
-            SvREFCNT_inc_simple_void_NN(dim_sv);  // overcome sv_2mortal
-            if (SvIOK(dim_sv)) {
-               dim_ = SvIVX(dim_sv);
-            } else if (SvPOK(dim_sv) && SvCUR(dim_sv) > 0 && grok_number(SvPVX(dim_sv), SvCUR(dim_sv), &dim_val) == IS_NUMBER_IN_UV) {
-               dim_ = dim_val;
-            } else {
-               throw std::runtime_error("wrong " + AnyString(SvPVX(glue::Serializer_Sparse_dim_key), SvCUR(glue::Serializer_Sparse_dim_key)) + " element in sparse input");
+         if (is_magic) {
+            if (MAGIC* mg = mg_findext(arr_or_hash, PERL_MAGIC_ext, &glue::sparse_input_vtbl)) {
+               sparse_ = true;
+               arr_or_hash = mg->mg_obj;
+               size_ = AvFILLp(arr_or_hash)+1;
+               if (size_ > 0 && AvARRAY(arr_or_hash)[size_-1] == glue::Serializer_Sparse_dim_key) {
+                  dim_ = Int_from_perlIV(SvIVX(AvARRAY(arr_or_hash)[size_-2]), "sparse container dimension");
+                  size_ -= 2;
+               }
+               return;
             }
+         } else {
+            sparse_ = true;
+            if ((dim_sv = hv_delete_ent(hv, glue::Serializer_Sparse_dim_key, 0, SvSHARED_HASH(glue::Serializer_Sparse_dim_key)))) {
+               UV dim_val;
+               SvREFCNT_inc_simple_void_NN(dim_sv);  // overcome sv_2mortal
+               if (SvIOK(dim_sv)) {
+                  dim_ = Int_from_perlIV(SvIVX(dim_sv), "sparse container dimension");
+               } else if (SvPOK(dim_sv) && SvCUR(dim_sv) > 0 && grok_number(SvPVX(dim_sv), SvCUR(dim_sv), &dim_val) == IS_NUMBER_IN_UV) {
+                  dim_ = Int_from_perlIV((IV)dim_val, "sparse container dimension");
+               } else {
+                  throw std::runtime_error("wrong " + AnyString(SvPVX(glue::Serializer_Sparse_dim_key), SvCUR(glue::Serializer_Sparse_dim_key)) + " element in sparse input");
+               }
+            }
+            size_ = HvUSEDKEYS(hv);
+            hv_iterinit(hv);
+            if (!hv_iternext(hv)) i = size_;
+            return;
          }
-         size_ = HvUSEDKEYS(hv);
-         hv_iterinit(hv);
-         if (!hv_iternext(hv)) i = size_;
-         return;
       }
    }
    throw std::runtime_error("invalid list input: must be an array or hash");
@@ -287,11 +305,11 @@ bool ListValueInputBase::is_ordered() const
 SV* ListValueInputBase::get_first() const
 {
    dTHX;
-   if (is_ordered()) {
-      if (!sparse_representation()) {
+   if (SvTYPE(arr_or_hash) == SVt_PVAV) {
+      if (!sparse_) {
          return SvMAGICAL(arr_or_hash) ? *av_fetch((AV*)arr_or_hash, 0, FALSE) : AvARRAY(arr_or_hash)[0];
-      } else if (size_ > 2) {
-         return AvARRAY(arr_or_hash)[2];
+      } else if (size_ > 1) {
+         return AvARRAY(arr_or_hash)[1];
       } else {
          return nullptr;
       }
@@ -314,10 +332,10 @@ void ListValueInputBase::finish()
 SV* ListValueInputBase::get_next()
 {
    dTHX;
-   if (is_ordered()) {
+   if (SvTYPE(arr_or_hash) == SVt_PVAV) {
       SV* sv;
-      if (sparse_representation()) {
-         sv = AvARRAY(arr_or_hash)[i + 1];
+      if (sparse_) {
+         sv = AvARRAY(arr_or_hash)[i+1];
          i += 2;
       } else {
          sv = SvMAGICAL(arr_or_hash) ? *av_fetch((AV*)arr_or_hash, i++, FALSE) : AvARRAY(arr_or_hash)[i++];
@@ -329,14 +347,14 @@ SV* ListValueInputBase::get_next()
    return HeVAL(entry);
 }
 
-int ListValueInputBase::get_index() const
+Int ListValueInputBase::get_index() const
 {
-   if (!sparse_representation())
+   if (!sparse_)
       throw std::runtime_error("dense/sparse input mismatch");
-   if (is_ordered()) {
+   if (SvTYPE(arr_or_hash) == SVt_PVAV) {
       SV* ix_sv = AvARRAY(arr_or_hash)[i];
       if (SvIOK(ix_sv))
-         return SvIVX(ix_sv);
+         return Int_from_perlIV(SvIVX(ix_sv), "sparse element index");
       throw std::runtime_error("sparse input - invalid index");
    }
    dTHX;
@@ -346,7 +364,7 @@ int ListValueInputBase::get_index() const
    UV index;
    if (klen <= 0 || grok_number(key, klen, &index) != IS_NUMBER_IN_UV)
       throw std::runtime_error("sparse input - invalid index");
-   return index;
+   return Int_from_perlIV((IV)index, "sparse element index");
 }
 
 void ListValueInputBase::retrieve_key(std::string& dst) const
@@ -358,30 +376,24 @@ void ListValueInputBase::retrieve_key(std::string& dst) const
    dst.assign(key, klen);
 }
 
-Stack::Stack()
-{
-   dTHX;
-   pi=getTHX;
-}
+Stack::Stack() {}
 
 Stack::Stack(SV** start)
 {
    dTHX;
-   pi=getTHX;
-   PL_stack_sp=start;
+   PL_stack_sp = start;
 }
 
-Stack::Stack(int reserve)
+Stack::Stack(Int reserve)
 {
    dTHX;
-   pi=getTHX;
    PmStartFuncall(reserve);
    PUTBACK;
 }
 
 void Stack::push(SV* x) const
 {
-   dTHXa(pi);
+   dTHX;
    dSP;
    PUSHs(x);
    PUTBACK;
@@ -389,15 +401,15 @@ void Stack::push(SV* x) const
 
 void Stack::xpush(SV* x) const
 {
-   dTHXa(pi);
+   dTHX;
    dSP;
    XPUSHs(x);
    PUTBACK;
 }
 
-void Stack::extend(int n)
+void Stack::extend(Int n)
 {
-   dTHXa(pi);
+   dTHX;
    dSP;
    EXTEND(SP, n);
    PUTBACK;
@@ -405,7 +417,7 @@ void Stack::extend(int n)
 
 void Stack::push(const AnyString& s) const
 {
-   dTHXa(pi);
+   dTHX;
    dSP;
    mPUSHp(s.ptr, s.len);
    PUTBACK;
@@ -413,30 +425,37 @@ void Stack::push(const AnyString& s) const
 
 void Stack::cancel()
 {
-   dTHXa(pi);
+   dTHX;
    PmCancelFuncall;
 }
 
-void ListReturn::upgrade(int size)
+void ListReturn::upgrade(Int size)
 {
-   dTHXa(pi);
+   dTHX;
    dSP;
    EXTEND(SP, size);
 }
 
-long Value::int_value() const
+Int Value::Int_value() const
 {
    dTHX;
-   return SvIV(sv);
+   return Int_from_perlIV(SvIV(sv), "integral scalar value");
 }
 
-long Value::enum_value() const
+Int Value::enum_value(size_t s, bool expect_ref) const
 {
    dTHX;
-   return SvIV(SvRV(sv));
+   SV* val_sv = expect_ref ? SvRV(sv) : sv;
+   const IV iv = SvIV(val_sv);
+   if (s < sizeof(IV)) {
+      const IV limit = IV(1) << s*8;
+      if (iv >= limit || iv <= -limit)
+         throw std::runtime_error("enumeration constant value is out of valid range");
+   }
+   return static_cast<Int>(iv);
 }
 
-double Value::float_value() const
+double Value::Float_value() const
 {
    dTHX;
    return SvNV(sv);
@@ -476,21 +495,21 @@ std::false_type* Value::retrieve(char& x) const
 {
    dTHX;
    if (SvPOK(sv)) {
-      x=*SvPVX(sv);
+      x = *SvPVX(sv);
    } else if (SvOK(sv)) {
       switch (classify_number()) {
       case number_is_int: {
-         const int ival=SvIV(sv);
-         if (ival>=0 && ival<=9)
-            x=ival+'0';
+         const IV ival = SvIV(sv);
+         if (ival >= 0 && ival <= 9)
+            x = static_cast<char>(ival + '0');
          else
             throw std::runtime_error("invalid value for an input character property");
          break;
       }
       case number_is_float: {
-         const double dval=SvNV(sv);
-         if (dval>=0 && dval<=9)
-            x=static_cast<int>(dval)+'0';
+         const double dval = SvNV(sv);
+         if (dval >= 0 && dval <= 9)
+            x = static_cast<char>(static_cast<int>(dval) + '0');
          else
             throw std::runtime_error("invalid value for an input character property");
          break;
@@ -498,10 +517,10 @@ std::false_type* Value::retrieve(char& x) const
       default:
          if (SvROK(sv) && !SvAMAGIC(sv))
             throw std::runtime_error("invalid value for an input character property");
-         x=*SvPV_nolen(sv);
+         x = *SvPV_nolen(sv);
       }
    } else {
-      x=0;
+      x = 0;
    }
    return nullptr;
 }
@@ -527,7 +546,7 @@ Value::number_flags Value::classify_number() const
       return number_is_float;
    if (flags & SVf_POK) {
       if (SvCUR(sv) > 0) {
-         flags=looks_like_number(sv);
+         flags = looks_like_number(sv);
          if (flags & (IS_NUMBER_NOT_INT | IS_NUMBER_GREATER_THAN_UV_MAX))
             return number_is_float;
          if (flags & IS_NUMBER_IN_UV)
@@ -565,9 +584,9 @@ std::false_type* Value::retrieve(bool& x) const
 {
    dTHX;
    if (SvPOK(sv) && SvCUR(sv)==5 && !strcmp(SvPVX(sv), "false")) {
-      x=false;
+      x = false;
    } else {
-      x=SvTRUE(sv);
+      x = SvTRUE(sv);
    }
    return nullptr;
 }
@@ -577,16 +596,16 @@ std::false_type* Value::retrieve(double& x) const
    dTHX;
    switch (classify_number()) {
    case number_is_int:
-      x=double(SvIV(sv));
+      x = static_cast<double>(SvIV(sv));
       break;
    case number_is_float:
-      x=SvNV(sv);
+      x = SvNV(sv);
       break;
    case number_is_object:
-      x=Scalar::convert_to_float(sv);
+      x = Scalar::convert_to_Float(sv);
       break;
    case number_is_zero:
-      x=0;
+      x = 0;
       break;
    default:
       throw std::runtime_error("invalid value for an input floating-point property");
@@ -604,7 +623,7 @@ std::false_type* Value::retrieve(Array& x) const
    } else if (options * ValueFlags::allow_undef) {
       x.clear();
    } else {
-      throw undefined();
+      throw Undefined();
    }
    return nullptr;
 }
@@ -616,12 +635,12 @@ bool Value::is_plain_text(bool expect_numeric_scalar) const
    } else if (SvROK(sv) && SvOBJECT(SvRV(sv))) {
       dTHX;
       SV* type;
-      if (sv_derived_from(sv, "Polymake::Core::Object")) {
+      if (sv_derived_from(sv, "Polymake::Core::BigObject")) {
          PmStartFuncall(1);
          PUSHs(sv);
          PUTBACK;
          type=glue::call_method_scalar(aTHX_ "type");
-      } else if (sv_derived_from(sv, "Polymake::Core::ObjectType")) {
+      } else if (sv_derived_from(sv, "Polymake::Core::BigObjectType")) {
          type=sv;
       } else {
          return false;
@@ -651,14 +670,14 @@ Value::canned_data_t Value::get_canned_data(SV* sv_arg) noexcept
    return { nullptr, nullptr, false };
 }
 
-int Value::get_canned_dim(bool tell_size_if_dense) const
+Int Value::get_canned_dim(bool tell_size_if_dense) const
 {
    if (SvROK(sv)) {
       MAGIC* mg;
-      SV* obj=SvRV(sv);
-      if (SvOBJECT(obj) && (mg=glue::get_cpp_magic(obj))) {
-         const glue::container_vtbl* t=(const glue::container_vtbl*)mg->mg_virtual;
-         if (((t->flags & ClassFlags::kind_mask) == ClassFlags::is_container) && t->own_dimension==1) {
+      SV* obj = SvRV(sv);
+      if (SvOBJECT(obj) && (mg = glue::get_cpp_magic(obj))) {
+         const glue::container_vtbl* t = static_cast<const glue::container_vtbl*>(mg->mg_virtual);
+         if (((t->flags & ClassFlags::kind_mask) == ClassFlags::is_container) && t->own_dimension == 1) {
             if (tell_size_if_dense || t->flags * ClassFlags::is_sparse_container)
                return (t->size)(mg->mg_ptr);
          }
@@ -667,20 +686,13 @@ int Value::get_canned_dim(bool tell_size_if_dense) const
    return -1;
 }
 
-Value::NoAnchors Value::put_val(long x, int)
+Value::NoAnchors Value::put_val(Int x, int)
 {
    dTHX;
    sv_setiv(sv, x);
    return NoAnchors();
 }
 
-Value::NoAnchors Value::put_val(unsigned long x, int)
-{
-   dTHX;
-   sv_setuv(sv, x);
-   return NoAnchors();
-}
-
 Value::NoAnchors Value::put_val(bool x, int)
 {
    dTHX;
@@ -695,7 +707,7 @@ Value::NoAnchors Value::put_val(double x, int)
    return NoAnchors();
 }
 
-Value::NoAnchors Value::put_val(const undefined&, int)
+Value::NoAnchors Value::put_val(const Undefined&, int)
 {
    dTHX;
    sv_setsv(sv, &PL_sv_undef);
@@ -722,41 +734,34 @@ void Value::set_string_value(const char* x, size_t l)
 
 namespace {
 
-inline
-Value::Anchor* finalize_primitive_ref(pTHX_ const Value& v, const char* xptr, SV* descr, int n_anchors, bool take_ref)
+Value::Anchor* finalize_primitive_ref(pTHX_ const Value& v, const char* xptr, SV* descr, int n_anchors)
 {
-   if (take_ref) {
-      MAGIC* mg=glue::upgrade_to_builtin_magic_sv(aTHX_ v.get(), descr, n_anchors);
-      mg->mg_ptr=(char*)xptr;
-      mg->mg_flags |= uint8_t(v.get_flags() & ValueFlags::read_only);
-      return n_anchors ? glue::MagicAnchors::first(mg) : nullptr;
-   } else {
-      MAGIC* mg=glue::upgrade_to_builtin_magic_sv(aTHX_ v.get(), descr, 0);
-      mg->mg_flags |= uint8_t(ValueFlags::read_only);
-      return nullptr;
-   }
+   MAGIC* mg = glue::upgrade_to_builtin_magic_sv(aTHX_ v.get(), descr, n_anchors);
+   mg->mg_ptr = (char*)xptr;
+   set_bit_flags(mg->mg_flags, v.get_flags() & ValueFlags::read_only);
+   return n_anchors ? glue::MagicAnchors::first(mg) : nullptr;
 }
 
 }
 
 void Value::Anchor::store(SV* sv) noexcept
 {
-   if (SvROK(sv)) sv=SvRV(sv);
-   stored=SvREFCNT_inc_simple_NN(sv);
+   if (SvROK(sv)) sv = SvRV(sv);
+   stored = SvREFCNT_inc_simple_NN(sv);
 }
 
 std::pair<void*, Value::Anchor*> Value::allocate_canned(SV* descr, int n_anchors) const
 {
    dTHX;
-   MAGIC* mg=glue::allocate_canned_magic(aTHX_ sv, descr, options | ValueFlags::alloc_magic, n_anchors);
+   MAGIC* mg = glue::allocate_canned_magic(aTHX_ sv, descr, options | ValueFlags::alloc_magic, n_anchors);
    mg->mg_flags |= MGf_GSKIP;    // if the following constructor call dies with an exception, destroy_canned won't try to delete the non-existent object
    return { mg->mg_ptr, n_anchors ? glue::MagicAnchors::first(mg) : nullptr };
 }
 
 void Value::mark_canned_as_initialized()
 {
-   MAGIC* mg=SvMAGIC(SvRV(sv));
-   mg->mg_flags &= ~MGf_GSKIP;    // confirm that the canned object is fully initialized
+   MAGIC* mg = SvMAGIC(SvRV(sv));
+   clear_bit_flags(mg->mg_flags, MGf_GSKIP);    // confirm that the canned object is fully initialized
 }
 
 SV* Value::get_constructed_canned()
@@ -768,66 +773,45 @@ SV* Value::get_constructed_canned()
 Value::Anchor* Value::store_canned_ref_impl(void* val, SV* descr, ValueFlags flags, int n_anchors) const
 {
    dTHX;
-   MAGIC* mg=glue::allocate_canned_magic(aTHX_ sv, descr, flags, n_anchors);
-   mg->mg_ptr=(char*)val;
+   MAGIC* mg = glue::allocate_canned_magic(aTHX_ sv, descr, flags, n_anchors);
+   mg->mg_ptr = (char*)val;
    return n_anchors ? glue::MagicAnchors::first(mg) : nullptr;
 }
 
-Value::Anchor* Value::store_primitive_ref(const bool& x, SV* descr, int n_anchors, bool take_ref)
+Value::Anchor* Value::store_primitive_ref(const bool& x, SV* descr, int n_anchors)
 {
    dTHX;
-   sv_upgrade(sv, SVt_PVLV);
+   (void)SvUPGRADE(sv, SVt_PVLV);
+   sv_setsv(sv, x ? &PL_sv_yes : &PL_sv_no);
+   return finalize_primitive_ref(aTHX_ *this, reinterpret_cast<const char*>(&x), descr, n_anchors);
+}
+Value::Anchor* Value::store_primitive_ref(const Int& x, SV* descr, int n_anchors)
+{
+   dTHX;
+   (void)SvUPGRADE(sv, SVt_PVLV);
    sv_setiv(sv, x);
-   return finalize_primitive_ref(aTHX_ *this, reinterpret_cast<const char*>(&x), descr, n_anchors, take_ref);
+   return finalize_primitive_ref(aTHX_ *this, reinterpret_cast<const char*>(&x), descr, n_anchors);
 }
-Value::Anchor* Value::store_primitive_ref(const int& x, SV* descr, int n_anchors, bool take_ref)
+Value::Anchor* Value::store_primitive_ref(const double& x, SV* descr, int n_anchors)
 {
    dTHX;
-   sv_upgrade(sv, SVt_PVLV);
-   sv_setiv(sv, x);
-   return finalize_primitive_ref(aTHX_ *this, reinterpret_cast<const char*>(&x), descr, n_anchors, take_ref);
-}
-Value::Anchor* Value::store_primitive_ref(const unsigned int& x, SV* descr, int n_anchors, bool take_ref)
-{
-   dTHX;
-   sv_upgrade(sv, SVt_PVLV);
-   sv_setuv(sv, x);
-   return finalize_primitive_ref(aTHX_ *this, reinterpret_cast<const char*>(&x), descr, n_anchors, take_ref);
-}
-Value::Anchor* Value::store_primitive_ref(const long& x, SV* descr, int n_anchors, bool take_ref)
-{
-   dTHX;
-   sv_upgrade(sv, SVt_PVLV);
-   sv_setiv(sv, x);
-   return finalize_primitive_ref(aTHX_ *this, reinterpret_cast<const char*>(&x), descr, n_anchors, take_ref);
-}
-Value::Anchor* Value::store_primitive_ref(const unsigned long& x, SV* descr, int n_anchors, bool take_ref)
-{
-   dTHX;
-   sv_upgrade(sv, SVt_PVLV);
-   sv_setuv(sv, x);
-   return finalize_primitive_ref(aTHX_ *this, reinterpret_cast<const char*>(&x), descr, n_anchors, take_ref);
-}
-Value::Anchor* Value::store_primitive_ref(const double& x, SV* descr, int n_anchors, bool take_ref)
-{
-   dTHX;
-   sv_upgrade(sv, SVt_PVLV);
+   (void)SvUPGRADE(sv, SVt_PVLV);
    sv_setnv(sv, x);
-   return finalize_primitive_ref(aTHX_ *this, reinterpret_cast<const char*>(&x), descr, n_anchors, take_ref);
+   return finalize_primitive_ref(aTHX_ *this, reinterpret_cast<const char*>(&x), descr, n_anchors);
 }
-Value::Anchor* Value::store_primitive_ref(const std::string& x, SV* descr, int n_anchors, bool take_ref)
+Value::Anchor* Value::store_primitive_ref(const std::string& x, SV* descr, int n_anchors)
 {
    dTHX;
-   sv_upgrade(sv, SVt_PVLV);
+   (void)SvUPGRADE(sv, SVt_PVLV);
    sv_setpvn(sv, x.c_str(), x.size());
-   return finalize_primitive_ref(aTHX_ *this, reinterpret_cast<const char*>(&x), descr, n_anchors, take_ref);
+   return finalize_primitive_ref(aTHX_ *this, reinterpret_cast<const char*>(&x), descr, n_anchors);
 }
 
 istream::istream(SV* sv)
    : BufferHolder<istreambuf>(sv), std::istream(&my_buf)
 {
    exceptions(failbit | badbit);
-   if (SvCUR(sv)==0)
+   if (SvCUR(sv) == 0)
       setstate(eofbit);
 }
 
@@ -842,8 +826,8 @@ istreambuf::istreambuf(SV* sv)
    if (__builtin_expect(SvROK(sv) && !SvAMAGIC(sv), 0))
       throw std::runtime_error("invalid value for an input property");
    size_t l;
-   char* p=SvPV(sv,l);
-   setg(p,p,p+l);
+   char* p = SvPV(sv, l);
+   setg(p, p, p+l);
 }
 
 ostreambuf::ostreambuf(SV* sv)
@@ -851,7 +835,7 @@ ostreambuf::ostreambuf(SV* sv)
 {
    dTHX;
    sv_setpvn(sv, "", 0);
-   char* p=SvGROW(sv,24);
+   char* p = SvGROW(sv,24);
    setp(p, p+23);
 }
 
@@ -864,19 +848,19 @@ ostreambuf::~ostreambuf()
 ostreambuf::int_type ostreambuf::overflow(int_type c)
 {
    dTHX;
-   size_t l=pptr()-pbase();
-   SvCUR_set(val,l);
-   char* p=SvGROW(val, l+513);
+   size_t l = pptr() - pbase();
+   SvCUR_set(val, l);
+   char* p = SvGROW(val, l+513);
    setp(p, p+l+512);
-   pbump(l);
-   if (!traits_type::eq(c, traits_type::eof())) {
-      *pptr()=c;
+   static_cast<OutCharBuffer*>(static_cast<std::streambuf*>(this))->pbump(l);
+   if (!traits_type::eq_int_type(c, traits_type::eof())) {
+      *pptr() = traits_type::to_char_type(c);
       pbump(1);
    }
    return traits_type::not_eof(c);
 }
 
-undefined::undefined()
+Undefined::Undefined()
    : std::runtime_error("unexpected undefined value of an input property") {}
 
 } }
